#
# Copyright (c) 2017-2024 AutoDeployAI
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
PyPMML is the Python API for PMML4S.
"""

from pypmml.model import Model
from pypmml.jvm import PMMLError
from pypmml.base import PMMLContext
from pypmml.version import __version__

__all__ = ['Model', 'PMMLError', 'PMMLContext']
